==> tengits_widgets-0.0.4.tar.gz: lib/selection/t_more_selection_page.dart <==
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:tengits_widgets/line/t_line.dart';
import 'package:tengits_widgets/toast/t_toast.dart';
import 'package:tengits_widgets/utils/multi_click_util.dart';

import 'selection_common_entity.dart';
import 'selection_utils.dart';
import 'selection_view.dart';
import 't_selection_config.dart';
import 'widget/t_selection_more_item_widget.dart';

/// 更多的多选页面
/// 展示的内容：
///         1：以纯标签的形式展示筛选条件 比如：朝向
///         2：以可点击的layout 展示跳转至列表页面 比如：商圈
///         3：以标签和自定义的输入展示筛选条件 比如：面积
///
/// 筛选条件的单选多选判定规则是：父节点的 type 决定子节点的单选多选类型
///                          子节点的 type 决定了自己的展示UI
/// 比如 楼层，楼层节点的type是radio，那么 一层、二层都是 只能选中一个
///                               如果他的某个子节点是range type， 该子节点的展示是自定义输入
///
///
/// 参考[TSelectionEntity]和[TSelectionView]
class TMoreSelectionPage extends StatefulWidget {
  final TSelectionEntity entityData;
  final Function(TSelectionEntity)? confirmCallback;
  final TOnCustomFloatingLayerClick? onCustomFloatingLayerClick;
  final TSelectionConfig themeData;

  const TMoreSelectionPage({Key? key, required this.entityData, this.confirmCallback, this.onCustomFloatingLayerClick, required this.themeData}) : super(key: key);

  @override
  _TMoreSelectionPageState createState() => _TMoreSelectionPageState();
}

class _TMoreSelectionPageState extends State<TMoreSelectionPage> with SingleTickerProviderStateMixin {
  final List<TSelectionEntity> _originalSelectedItemsList = [];
  late AnimationController _controller;
  late Animation<Offset> _animation;
  final StreamController<ClearEvent> _clearController = StreamController.broadcast();
  bool isValid = true;

  @override
  void initState() {
    super.initState();
    _controller = AnimationController(
      duration: const Duration(milliseconds: 300),
      vsync: this,
    );
    _animation = Tween(end: Offset.zero, begin: const Offset(1.0, 0.0)).animate(_controller);
    _controller.forward();

    _originalSelectedItemsList.addAll(widget.entityData.allSelectedList());
    for (TSelectionEntity entity in _originalSelectedItemsList) {
      entity.isSelected = true;
      if (entity.customMap != null) {
        // originalCustomMap 是用来存临时状态数据, customMap 用来展示 ui
        entity.originalCustomMap = Map.from(entity.customMap!);
      }
    }
  }

  /// 页面结构：左侧的透明黑 + 右侧宽为300的内容区域
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: const Color(0x660c0c0c),
      body: Row(
        children: <Widget>[
          _buildLeftSlide(context),
          AnimatedBuilder(
            animation: _controller,
            builder: (context, child) {
              return SlideTransition(
                position: _animation,
                child: child,
              );
            },
            child: _buildRightSlide(context),
          )
        ],
      ),
      //为了解决 键盘抬起按钮的问题 将按钮移动到 此区域
      bottomNavigationBar: SizedBox(
        height: 80 + _getBottomAreaHeight(),
        child: Row(
          children: <Widget>[
            _buildLeftSlide(context),
            AnimatedBuilder(
              animation: _controller,
              builder: (context, child) {
                return SlideTransition(
                  position: _animation,
                  child: child,
                );
              },
              child: Container(
                width: 300,
                color: Colors.white,
                child: Column(
                  children: <Widget>[
                    const TLine(),
                    Padding(
                      padding: const EdgeInsets.only(top: 14),
                      child: _buildBottomButtons(),
                    ),
                  ],
                ),
              ),
            )
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    _controller.dispose();
    _clearController.close();
  }

  /// 左侧为透明黑，点击直接退出页面
  Widget _buildLeftSlide(BuildContext context) {
    return Expanded(
      child: GestureDetector(
        onTap: () {
          SelectionUtil.resetSelectionDatas(widget.entityData);
          //把数据还原
          for (var data in _originalSelectedItemsList) {
            data.isSelected = true;
            if (data.customMap != null) {
              // originalCustomMap 是用来存临时状态数据, customMap 用来展示 ui
              data.customMap = <String, String>{};
              data.originalCustomMap.forEach((key, value) {
                data.customMap![key.toString()] = value.toString();
              });
            }
          }
          Navigator.maybePop(context);
        },
        child: Container(
          color: Colors.transparent,
        ),
      ),
    );
  }

  /// 右侧为内容区域：标题+更多+筛选项的列表 + 底部按钮区域
  Widget _buildRightSlide(BuildContext context) {
    return Container(
      width: 300,
      color: Colors.white,
      child: Padding(
        padding: const EdgeInsets.only(top: 0),
        child: _buildSelectionListView(),
      ),
    );
  }

  /// 标题+筛选条件的 列表
  Widget _buildSelectionListView() {
    return ListView.builder(
      itemBuilder: (context, index) {
        return TMoreSelectionWidget(clearController: _clearController, selectionEntity: widget.entityData.children[index], onCustomFloatingLayerClick: widget.onCustomFloatingLayerClick, themeData: widget.themeData);
      },
      itemCount: widget.entityData.children.length,
    );
  }

  /// 清空筛选项 + 确定按钮
  Widget _buildBottomButtons() {
    return MoreBottomSelectionWidget(
      entity: widget.entityData,
      themeData: widget.themeData,
      clearCallback: () {
        setState(() {
          _clearController.add(ClearEvent());
          _clearUIData(widget.entityData);
        });
      },
      conformCallback: (data) {
        checkValue(data);
        if (!isValid) {
          isValid = true;
          return;
        }

        for (var data in widget.entityData.children) {
          if (data.selectedList().isNotEmpty) {
            data.isSelected = true;
          } else {
            data.isSelected = false;
          }
        }
        if (widget.confirmCallback != null) {
          widget.confirmCallback!(data);
        }
        Navigator.of(context).pop();
      },
    );
  }

  //清空UI效果
  void _clearUIData(TSelectionEntity entity) {
    entity.isSelected = false;
    entity.customMap = <String, String>{};
    if (TSelectionFilterType.range == entity.filterType) {
      entity.title = '';
    }
    for (TSelectionEntity subEntity in entity.children) {
      _clearUIData(subEntity);
    }
  }

  void checkValue(TSelectionEntity entity) {
    clearSelectedEntity();
  }

  void clearSelectedEntity() {
    List<TSelectionEntity> tmp = [];
    TSelectionEntity node = widget.entityData;
    tmp.add(node);
    while (tmp.isNotEmpty) {
      node = tmp.removeLast();
      if (node.isSelected && (node.filterType == TSelectionFilterType.range || node.filterType == TSelectionFilterType.dateRange || node.filterType == TSelectionFilterType.dateRangeCalendar)) {
        if (node.customMap != null && !SelectionUtil.isEmpty(node.customMap!['min']) && !SelectionUtil.isEmpty(node.customMap!['max'])) {
          if (!node.isValidRange()) {
            isValid = false;
            if (node.filterType == TSelectionFilterType.range) {
              TToast.show('您输入的区间有误', context);
            } else if (node.filterType == TSelectionFilterType.dateRange || node.filterType == TSelectionFilterType.dateRangeCalendar) {
              TToast.show('您输入的区间有误', context);
            }
            return;
          }
        } else {
          node.isSelected = false;
        }
      }
      for (var data in node.children) {
        tmp.add(data);
      }
    }
  }

  double _getBottomAreaHeight() {
    return MediaQuery.of(context).padding.bottom;
  }
}

/// 底部的重置+确定
class MoreBottomSelectionWidget extends StatelessWidget {
  final TSelectionEntity entity;
  final VoidCallback? clearCallback;
  final Function(TSelectionEntity)? conformCallback;
  final TSelectionConfig themeData;

  const MoreBottomSelectionWidget({
    Key? key,
    required this.entity,
    this.clearCallback,
    this.conformCallback,
    required this.themeData,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        GestureDetector(
          onTap: () {
            if (clearCallback != null) {
              clearCallback!();
            }
          },
          child: Container(
            margin: const EdgeInsets.only(left: 20, right: 20),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: const <Widget>[
                SizedBox(
                  height: 24,
                  width: 24,
                  child: Icon(Icons.reset_tv),
                ),
                Text(
                  '重置',
                  style: TextStyle(
                    color: Color(0xFF666666),
                    fontSize: 12.0,
                  ),
                )
              ],
            ),
          ),
        ),
        Expanded(
            child: GestureDetector(
          onTap: () {
            if (MultiClickUtils.isMultiClick()) {
              return;
            }
            if (conformCallback != null) {
              conformCallback!(entity);
            }
          },
          child: Container(
            alignment: Alignment.center,
            decoration: const BoxDecoration(
              color: Color(0xFF0984F9),
              borderRadius: BorderRadius.all(Radius.circular(6)),
            ),
            constraints: const BoxConstraints.tightFor(width: double.infinity, height: 50),
            padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 8),
            child: const Text(
              "确定",
              overflow: TextOverflow.ellipsis,
              maxLines: 1,
              style: TextStyle(
                fontSize: 16.0,
                color: Colors.white,
                fontWeight: FontWeight.bold,
              ),
            ),
          ),
        )),
      ],
    );
  }
}

//用于处理 重置事件
class ClearEvent {}

==> tengits_widgets-0.0.10.tar.gz: lib/selection/t_more_selection_page.dart <==
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:tengits_widgets/line/t_line.dart';
import 'package:tengits_widgets/toast/t_toast.dart';
import 'package:tengits_widgets/utils/multi_click_util.dart';

import 'selection_common_entity.dart';
import 'selection_utils.dart';
import 'selection_view.dart';
import 't_selection_config.dart';
import 'widget/t_selection_more_item_widget.dart';

/// 更多的多选页面
/// 展示的内容：
///         1：以纯标签的形式展示筛选条件 比如：朝向
///         2：以可点击的layout 展示跳转至列表页面 比如：商圈
///         3：以标签和自定义的输入展示筛选条件 比如：面积
///
/// 筛选条件的单选多选判定规则是：父节点的 type 决定子节点的单选多选类型
///                          子节点的 type 决定了自己的展示UI
/// 比如 楼层，楼层节点的type是radio，那么 一层、二层都是 只能选中一个
///                               如果他的某个子节点是range type， 该子节点的展示是自定义输入
///
///
/// 参考[TSelectionEntity]和[TSelectionView]
class TMoreSelectionPage extends StatefulWidget {
  final TSelectionEntity entityData;
  final Function(TSelectionEntity)? confirmCallback;
  final TOnCustomFloatingLayerClick? onCustomFloatingLayerClick;
  final TSelectionConfig themeData;

  const TMoreSelectionPage({Key? key, required this.entityData, this.confirmCallback, this.onCustomFloatingLayerClick, required this.themeData}) : super(key: key);

  @override
  TMoreSelectionPageState createState() => TMoreSelectionPageState();
}

class TMoreSelectionPageState extends State<TMoreSelectionPage> with SingleTickerProviderStateMixin {
  final List<TSelectionEntity> _originalSelectedItemsList = [];
  late AnimationController _controller;
  late Animation<Offset> _animation;
  final StreamController<ClearEvent> _clearController = StreamController.broadcast();
  bool isValid = true;

  @override
  void initState() {
    super.initState();
    _controller = AnimationController(
      duration: const Duration(milliseconds: 300),
      vsync: this,
    );
    _animation = Tween(end: Offset.zero, begin: const Offset(1.0, 0.0)).animate(_controller);
    _controller.forward();

    _originalSelectedItemsList.addAll(widget.entityData.allSelectedList());
    for (TSelectionEntity entity in _originalSelectedItemsList) {
      entity.isSelected = true;
      if (entity.customMap != null) {
        // originalCustomMap 是用来存临时状态数据, customMap 用来展示 ui
        entity.originalCustomMap = Map.from(entity.customMap!);
      }
    }
  }

  /// 页面结构：左侧的透明黑 + 右侧宽为300的内容区域
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: const Color(0x660c0c0c),
      body: Row(
        children: <Widget>[
          _buildLeftSlide(context),
          AnimatedBuilder(
            animation: _controller,
            builder: (context, child) {
              return SlideTransition(
                position: _animation,
                child: child,
              );
            },
            child: _buildRightSlide(context),
          )
        ],
      ),
      //为了解决 键盘抬起按钮的问题 将按钮移动到 此区域
      bottomNavigationBar: SizedBox(
        height: 80 + _getBottomAreaHeight(),
        child: Row(
          children: <Widget>[
            _buildLeftSlide(context),
            AnimatedBuilder(
              animation: _controller,
              builder: (context, child) {
                return SlideTransition(
                  position: _animation,
                  child: child,
                );
              },
              child: Container(
                width: 300,
                color: Colors.white,
                child: Column(
                  children: <Widget>[
                    const TLine(),
                    Padding(
                      padding: const EdgeInsets.only(top: 14),
                      child: _buildBottomButtons(),
                    ),
                  ],
                ),
              ),
            )
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    _controller.dispose();
    _clearController.close();
  }

  /// 左侧为透明黑，点击直接退出页面
  Widget _buildLeftSlide(BuildContext context) {
    return Expanded(
      child: GestureDetector(
        onTap: () {
          SelectionUtil.resetSelectionDatas(widget.entityData);
          //把数据还原
          for (var data in _originalSelectedItemsList) {
            data.isSelected = true;
            if (data.customMap != null) {
              // originalCustomMap 是用来存临时状态数据, customMap 用来展示 ui
              data.customMap = <String, String>{};
              data.originalCustomMap.forEach((key, value) {
                data.customMap![key.toString()] = value.toString();
              });
            }
          }
          Navigator.maybePop(context);
        },
        child: Container(
          color: Colors.transparent,
        ),
      ),
    );
  }

  /// 右侧为内容区域：标题+更多+筛选项的列表 + 底部按钮区域
  Widget _buildRightSlide(BuildContext context) {
    return Container(
      width: 300,
      color: Colors.white,
      child: Padding(
        padding: const EdgeInsets.only(top: 0),
        child: _buildSelectionListView(),
      ),
    );
  }

  /// 标题+筛选条件的 列表
  Widget _buildSelectionListView() {
    return ListView.builder(
      itemBuilder: (context, index) {
        return TMoreSelectionWidget(clearController: _clearController, selectionEntity: widget.entityData.children[index], onCustomFloatingLayerClick: widget.onCustomFloatingLayerClick, themeData: widget.themeData);
      },
      itemCount: widget.entityData.children.length,
    );
  }

  /// 清空筛选项 + 确定按钮
  Widget _buildBottomButtons() {
    return MoreBottomSelectionWidget(
      entity: widget.entityData,
      themeData: widget.themeData,
      clearCallback: () {
        setState(() {
          _clearController.add(ClearEvent());
          _clearUIData(widget.entityData);
        });
      },
      conformCallback: (data) {
        checkValue(data);
        if (!isValid) {
          isValid = true;
          return;
        }

        for (var data in widget.entityData.children) {
          if (data.selectedList().isNotEmpty) {
            data.isSelected = true;
          } else {
            data.isSelected = false;
          }
        }
        if (widget.confirmCallback != null) {
          widget.confirmCallback!(data);
        }
        Navigator.of(context).pop();
      },
    );
  }

  //清空UI效果
  void _clearUIData(TSelectionEntity entity) {
    entity.isSelected = false;
    entity.customMap = <String, String>{};
    if (TSelectionFilterType.range == entity.filterType) {
      entity.title = '';
    }
    for (TSelectionEntity subEntity in entity.children) {
      _clearUIData(subEntity);
    }
  }

  void checkValue(TSelectionEntity entity) {
    clearSelectedEntity();
  }

  void clearSelectedEntity() {
    List<TSelectionEntity> tmp = [];
    TSelectionEntity node = widget.entityData;
    tmp.add(node);
    while (tmp.isNotEmpty) {
      node = tmp.removeLast();
      if (node.isSelected && (node.filterType == TSelectionFilterType.range || node.filterType == TSelectionFilterType.dateRange || node.filterType == TSelectionFilterType.dateRangeCalendar)) {
        if (node.customMap != null && !SelectionUtil.isEmpty(node.customMap!['min']) && !SelectionUtil.isEmpty(node.customMap!['max'])) {
          if (!node.isValidRange()) {
            isValid = false;
            if (node.filterType == TSelectionFilterType.range) {
              TToast.show('您输入的区间有误', context);
            } else if (node.filterType == TSelectionFilterType.dateRange || node.filterType == TSelectionFilterType.dateRangeCalendar) {
              TToast.show('您输入的区间有误', context);
            }
            return;
          }
        } else {
          node.isSelected = false;
        }
      }
      for (var data in node.children) {
        tmp.add(data);
      }
    }
  }

  double _getBottomAreaHeight() {
    return MediaQuery.of(context).padding.bottom;
  }
}

/// 底部的重置+确定
class MoreBottomSelectionWidget extends StatelessWidget {
  final TSelectionEntity entity;
  final VoidCallback? clearCallback;
  final Function(TSelectionEntity)? conformCallback;
  final TSelectionConfig themeData;

  const MoreBottomSelectionWidget({
    Key? key,
    required this.entity,
    this.clearCallback,
    this.conformCallback,
    required this.themeData,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        GestureDetector(
          onTap: () {
            if (clearCallback != null) {
              clearCallback!();
            }
          },
          child: Container(
            margin: const EdgeInsets.only(left: 20, right: 20),
            child: const Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                SizedBox(
                  height: 24,
                  width: 24,
                  child: Icon(Icons.reset_tv),
                ),
                Text(
                  '重置',
                  style: TextStyle(
                    color: Color(0xFF666666),
                    fontSize: 12.0,
                  ),
                )
              ],
            ),
          ),
        ),
        Expanded(
            child: GestureDetector(
          onTap: () {
            if (MultiClickUtils.isMultiClick()) {
              return;
            }
            if (conformCallback != null) {
              conformCallback!(entity);
            }
          },
          child: Container(
            alignment: Alignment.center,
            decoration: const BoxDecoration(
              color: Color(0xFF0984F9),
              borderRadius: BorderRadius.all(Radius.circular(6)),
            ),
            constraints: const BoxConstraints.tightFor(width: double.infinity, height: 50),
            padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 8),
            child: const Text(
              "确定",
              overflow: TextOverflow.ellipsis,
              maxLines: 1,
              style: TextStyle(
                fontSize: 16.0,
                color: Colors.white,
                fontWeight: FontWeight.bold,
              ),
            ),
          ),
        )),
      ],
    );
  }
}

//用于处理 重置事件
class ClearEvent {}
